<?php
namespace ExcitedFree\App\Model;

use LooksAwesome\Common\Post;
use ExcitedFree\App\Shortcode;

class Testimonial extends Post {
    public static $model_name = 'testimonial';
    public static $post_type = 'aet-testimonial';

    public static $validation_rules = array(
        'name' => array(
            'rule' => 'required'
        )
    );

    public static $fill_rules = array(
        'foto'
    );

    public static $labels = array(
        'name'               => 'Testimonials',
        'singular_name'      => 'Testimonial',
        'menu_name'          => 'Testimonial',
        'name_admin_bar'     => 'Testimonia',
        'add_new'            => 'Add Testimonial',
        'add_new_item'       => '+ Add new testimonial',
        'new_item'           => 'New Testimonial',
        'edit_item'          => 'Edit Testimonial',
        'view_item'          => 'View Testimonial',
        'delete_item'        => 'Delete',
        'all_items'          => 'All Testimonials',
        'search_items'       => 'Search Testimonials',
        'parent_item_colon'  => 'Parent Testimonials:',
        'not_found'          => 'No Testimonials found.',
        'not_found_in_trash' => 'No Testimonials found in Trash.',
        'add_to_group'       => 'Add testimonial to group',
        'approve_item'       => 'Approve Testimonial'
    );

    public static $fields = array(
        'title', 'editor', 'thumbnail', 'excerpt'
    );

    protected static $defaults = array(
        'post_status' => 'publish',
        'name' => '',
        'category' => '',
        'short_text' => '',
        'full_text' => '',
        'custom_fields' => array(
            'created_from' => 'admin',
            'position' => '',
            'thumbnail_id' => 0,
            'rating' => 0,
            'rating_label' => '',
            'source_name' => '',
            'source_url' => '',
            'custom_url' => '',
            'email' => '',
            'phone' => '',
            'skype' => '',
            'location' => '',
            'facebook' => '',
            'twitter' => '',
            'linkedin' => '',
            'google' => '',
            'instagram' => '',
            'pinterest' => '',
            'youtube' => '',
            'vimeo' => '',
            'dribble' => '',
            'flickr' => '',
            'github' => '',
        )
    );

    protected static $rating_scales = array(
        '5s' => array(
            'label' => '&#9733;&#9733;&#9733;&#9733;&#9734;',
            'template' => array(
                'svg' => '<svg style="%1$s %2$s %3$s" class="svg-icon" viewBox="0 0 32 32"><use xmlns:xlink="http://www.w3.org/1999/xlink" xlink:href="#aet_icon-%4$s"></use></svg>',
                'base64' => '<div style="width: 24px;height: 24px; max-height: 24px;" class="svg-like svg-icon"><div style="%3$s %4$s %5$s"></div></div>'
            )
        ),
        '5d' => array(
            'label' => '1 - 5',
            //'template' => '<div class="rating-format-5d">%s / 5</div>'
            'template' => '<div class="rating-format-5d">%1.1f</div>'
        ),
        '10d' => array(
            'label' => '1 - 10',
            //'template' => '<div class="rating-format-10d">%s / 10</div>'
            'template' => '<div class="rating-format-10d">%s</div>'
        ),
        '100d' => array(
            'label' => '1 - 100',
            //'template' => '<div class="rating-format-100d">%s&#37;</div>'
            'template' => '<div class="rating-format-100d">%s</div>'
        )
    );

    /**
     * @return array
     */
    public static function prepareDefaults(){
        foreach(static::$defaults as $key => $value){
            $template = Template::get_template($key);
            static::$defaults['custom_fields']['styles'][$key] = $template[$value]['styles'];
        }
        return static::$defaults;
    }

    public function after_save($input){
        $thumbnail_id = $input['custom_fields']['thumbnail_id'];
        if(!empty($thumbnail_id)){
            set_post_thumbnail($this->id, $thumbnail_id);
        }else{
            delete_post_thumbnail($this->id);
        }
        wp_set_post_terms($this->id,  $input['category'], static::$taxonomy);
    }

    public function prepare_input($input){
        $custom_fields = $this->getDefault('custom_fields');
        $data = array(
            'ID' => $this->id,
            'post_type' => static::$post_type,
            'post_status' => isset($input['post_status']) ? $input['post_status'] : 'publish',
            'post_title' => $input['name'],
            'post_excerpt' => $input['short_text'],
            'post_content' => $input['full_text'],
            'custom_fields' => $this->assign_custom_fields($custom_fields, $input['custom_fields'])
        );

        return $data;
    }

    public function delete($force_delete = true){
        if(isset($this->post->groups)){
            foreach($groups = $this->post->groups as $id){
                $group = Group::find($id);
                $testimonials = $group->testimonials;
                if(($key = array_search($this->id, $testimonials)) !== false){
                    unset($testimonials[$key]);
                    update_post_meta($group->ID, 'testimonials', $testimonials);
                }
            }
        }
        wp_delete_post($this->id, $force_delete);
    }

    public static function get_photo($id, $preview){
        if(isset($id) && has_post_thumbnail($id)){
            $foto_id = get_post_thumbnail_id($id);
            $foto = wp_get_attachment_image_src($foto_id, 'large', false);
            $photo = $foto[0];
        }else{
            $base = plugin_dir_url(realpath(__DIR__ . '/..'));
            $photo = $base . 'public/img/default-avatar.jpg';
            if($preview) {
                $photo = $base . 'public/img/john-doe.jpg';
            }
        }
        return $photo;
    }

    public function getRatingHTML($args){
        $args['rating'] = $this->rating;
        return static::getRatingHTMLStatic($args);
    }

    public static function getRatingHTMLStatic($args){
        $rating = (int) esc_html($args['rating']);
        $scales = array_keys(self::$rating_scales);
        switch ($args['scale']) {
            case $scales[0]:
                if ($args['icon']) {
                    $size = preg_replace('/[^0-9]/', '', $args['size']);
                    $count = preg_replace('/[^0-9]/', '', $args['count']);
                    $size = (int) $size; // padding 1px both sides
                    $width = 'width:' . $size * $count * $rating / 100 . 'px';
                    $skill_value_width = 'width: ' . $size * $count . 'px';
                    $size = Shortcode::get_size($args['size']);
                    $data = array(
                        'svg' => array(
                            'width:' . $size . ';',
                            'height:' . $size . ';',
                            'color:' . $args['bar_color'] . ';',
                            $args['icon']
                        ),
                        'base64' => array(
                            'width:' . $size . ';',
                            'height:' . $size . ';',
                            'background-color:' . $args['bar_color'] , ';',
                            '-webkit-mask-image: url(\'' . $args['source'] . '\')',
                            'mask-image: url(\'' . $args['source'] . '\')'
                        )
                    );
                    if($args['source'] && $args['icon'] == 'replace'){
                        $rating_icon = vsprintf(self::$rating_scales[$scales[0]]['template']['base64'], $data['base64']);
                    }else{
                        $rating_icon = vsprintf(self::$rating_scales[$scales[0]]['template']['svg'], $data['svg']);
                    }

                    $rating_html = '<div class="rating-bar rating-background">';
                    for ($i = 0; $i < $count; $i++) {
                        $rating_html .= $rating_icon;
                    }
                    $rating_html .= '</div>';
                    $rating_html .= '<div class="rating-bar rating-fill" style="' . $width . '">';
                    for ($i = 0; $i < $count; $i++) {
                        $rating_html .= $rating_icon;
                    }
                    $rating_html .= '</div>';
                }

                $html = "<div class='skill-value rating-format-5s' style='$skill_value_width'>$rating_html</div>";

                return $html;
                break;
            case $scales[1]:
                $result = $rating / 20;
                return sprintf(self::$rating_scales[$scales[1]]['template'], $result);
                break;
            case $scales[2]:
                $result = $rating / 10;
                return sprintf(self::$rating_scales[$scales[2]]['template'], $result);
                break;
            case $scales[3]:
                return sprintf(self::$rating_scales[$scales[3]]['template'], $rating);
                break;
        }
    }

    public function getAllRatingHTML($args){
        $scales = array_keys(self::$rating_scales);
        $html = '';
        foreach ($scales as $scale){
            $args['scale'] = $scale;
            $html .= $this->getRatingHTML($args);
        }
        return $html;
    }

    /**
     * @return array
     */
    public static function getRatingScales()
    {
        return self::$rating_scales;
    }
} 